import 'package:flutter/material.dart';
import 'package:robomotion_app_lib/parser.dart';
import 'dart:convert';

class DialogBlock extends StatelessWidget {
  final String block;

  final Parser buttonParser;
  final Parser dividerParser;
  final Parser headerParser;
  final Parser imageParser;
  final Parser sectionParser;
  final Parser spacerParser;

  DialogBlock(this.block,
      {this.buttonParser = defaultButtonParser,
      this.spacerParser = defaultSpacerParser,
      this.sectionParser = defaultSectionParser,
      this.dividerParser = defaultDividerParser,
      this.headerParser = defaultHeaderParser,
      this.imageParser = defaultImageParser});

  List<Widget> renderBlock(BuildContext context) {
    var json = jsonDecode(this.block);
    List<Widget> widgets = [];

    for (var block in json["blocks"]) {
      var blockType = block["type"];
      Widget widget;
      switch (blockType) {
        case "section":
          {
            widget = renderSectionBlock(context, block);
          }
          break;

        case "actions":
          {
            widget = renderActionsBlock(context, block);
          }
          break;

        case "divider":
          {
            widget = renderDividerBlock(context, block);
          }
          break;

        case "image":
          {
            widget = renderImageBlock(context, block);
          }
          break;

        case "context":
          {
            widget = renderContextBlock(context, block);
          }
          break;

        case "header":
          {
            widget = renderHeaderBlock(context, block);
          }
          break;
      }

      if (widget != null) {
        widgets.add(widget);
      }
    }

    return widgets;
  }

  Widget renderSectionBlock(BuildContext context, block) {
    return defaultSectionParser.parse(block, context: context);
  }

  Widget renderActionsBlock(BuildContext context, block) {
    List elements = block["elements"];
    List<Widget> actionWidgets = [];

    int i = 0;
    for (Map element in elements) {
      switch (element["type"]) {
        case "button":
          var mL = i == 0 ? 0.0 : 8.0;
          var mR = i == elements.length - 1 ? 0.0 : 8.0;
          actionWidgets.add(buttonParser.parse(element,
              context: context, options: Options(margin: EdgeInsets.fromLTRB(mL, 8, mR, 8))));
          break;

        case "spacer":
          actionWidgets.add(spacerParser.parse(element, context: context));
          break;

        default:
          break;
      }
      i++;
    }

    return Row(
      children: actionWidgets,
    );
  }

  Widget renderDividerBlock(BuildContext context, block) {
    return dividerParser.parse(block, context: context);
  }

  Widget renderImageBlock(BuildContext context, block) {
    return imageParser.parse(block, context: context);
  }

  Widget renderContextBlock(BuildContext context, block) {
    return Container();
  }

  Widget renderHeaderBlock(BuildContext context, block) {
    return headerParser.parse(block, context: context);
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      padding: EdgeInsets.all(16),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: renderBlock(context),
      ),
    );
  }
}
